@@ -81,22 +81,23 @@ KMixD::KMixD(QObject* parent, const QList<QVariant>&) :
  */
 void KMixD::delayedInitialization()
 {
-	qCDebug(KMIX_LOG) << "kmixd: Delayed initialization running now";
+    qCDebug(KMIX_LOG) << "Delayed initialization running now";
    //initActions(); // init actions first, so we can use them in the loadConfig() already
    loadConfig(); // Load config before initMixer(), e.g. due to "MultiDriver" keyword
-   MixerToolBox::instance()->initMixer(m_multiDriverMode, m_backendFilter, m_hwInfoString, true);
-   KMixDeviceManager *theKMixDeviceManager = KMixDeviceManager::instance();
-   theKMixDeviceManager->initHotplug();
-   connect(theKMixDeviceManager, SIGNAL(plugged(const char*,QString,QString&)), SLOT (plugged(const char*,QString,QString&)) );
-   connect(theKMixDeviceManager, SIGNAL(unplugged(QString)), SLOT (unplugged(QString)) );
+   MixerToolBox::initMixer(m_multiDriverMode, m_backendFilter, true);
 
-    qCDebug(KMIX_LOG) << "kmixd: Delayed initialization done";
+   KMixDeviceManager *theKMixDeviceManager = KMixDeviceManager::instance();
+   connect(theKMixDeviceManager, &KMixDeviceManager::plugged, this, &KMixD::plugged);
+   connect(theKMixDeviceManager, &KMixDeviceManager::unplugged, this, &KMixD::unplugged);
+   theKMixDeviceManager->initHotplug();
+
+   qCDebug(KMIX_LOG) << "Delayed initialization done";
 }
 
 
 KMixD::~KMixD()
 {
-   MixerToolBox::instance()->deinitMixer();
+   MixerToolBox::deinitMixer();
 }
 
 
@@ -129,7 +130,7 @@ void KMixD::saveBaseConfig()
    if ( mdMaster ) {
       config.writeEntry( "MasterMixerDevice", mdMaster->id() );
    }
-   QString mixerIgnoreExpression = MixerToolBox::instance()->mixerIgnoreExpression();
+   QString mixerIgnoreExpression = MixerToolBox::mixerIgnoreExpression();
    config.writeEntry( "MixerIgnoreExpression", mixerIgnoreExpression );
 
    qCDebug(KMIX_LOG) << "Config (Base) saving done";
@@ -152,26 +153,27 @@ void KMixD::loadBaseConfig()
    Mixer::setGlobalMaster(mixerMasterCard, masterDev, true);
    QString mixerIgnoreExpression = config.readEntry( "MixerIgnoreExpression", "Modem" );
    m_backendFilter = config.readEntry<>( "Backends", QList<QString>() );
-   MixerToolBox::instance()->setMixerIgnoreExpression(mixerIgnoreExpression);
+   MixerToolBox::setMixerIgnoreExpression(mixerIgnoreExpression);
 }
 
 
-void KMixD::plugged( const char* driverName, const QString& /*udi*/, QString& dev)
+void KMixD::plugged(const char *driverName, const QString &udi, int dev)
 {
-//     qCDebug(KMIX_LOG) << "Plugged: dev=" << dev << "(" << driverName << ") udi=" << udi << "\n";
-    QString driverNameString;
-    driverNameString = driverName;
-    int devNum = dev.toInt();
-    Mixer *mixer = new Mixer( driverNameString, devNum );
-    if ( mixer != 0 ) {
-        qCDebug(KMIX_LOG) << "Plugged: dev=" << dev << "\n";
-        MixerToolBox::instance()->possiblyAddMixer(mixer);
+    qCDebug(KMIX_LOG) << "dev" << dev << "driver" << driverName << "udi" << udi;
+
+    Mixer *mixer = new Mixer(QString::fromLocal8Bit(driverName), dev);
+    if (mixer!=nullptr)
+    {
+        qCDebug(KMIX_LOG) << "adding mixer" << mixer->id() << mixer->readableName();
+        MixerToolBox::possiblyAddMixer(mixer);
     }
-
 }
 
-void KMixD::unplugged( const QString& udi)
+
+void KMixD::unplugged(const QString &udi)
 {
+    qCDebug(KMIX_LOG) << "udi" << udi;
+
 //     qCDebug(KMIX_LOG) << "Unplugged: udi=" <<udi << "\n";
     for (int i=0; i<Mixer::mixers().count(); ++i) {
         Mixer *mixer = (Mixer::mixers())[i];
@@ -181,7 +183,7 @@ void KMixD::unplugged( const QString& udi)
             //KMixToolBox::notification("MasterFallback", "aaa");
             bool globalMasterMixerDestroyed = ( mixer == Mixer::getGlobalMasterMixer() );
 
-            MixerToolBox::instance()->removeMixer(mixer);
+            MixerToolBox::removeMixer(mixer);
             // Check whether the Global Master disappeared, and select a new one if necessary
             shared_ptr<MixDevice> md = Mixer::getGlobalMasterMD();
             if ( globalMasterMixerDestroyed || md.get() == 0 ) {
